#!/usr/bin/python

import sys
import collections
import heapq
import Queue

stdin = sys.stdin

def topsort(n,edges):
    indegree = [ 0 for i in range(n) ]
    G = [ [] for i in range(n) ]
    for x,y,_,_ in edges:
        G[x].append(y)
        indegree[y] += 1
    topsort = []
    Q = Queue.Queue()
    for i in range(n):
        if indegree[i]==0:
            topsort.append(i)
            Q.put(i)
    while not Q.empty():
        x = Q.get()
        for y in G[x]:
            indegree[y] -= 1
            if indegree[y] == 0:
                topsort.append(y)
                Q.put(y)
    return topsort

def read_testcase():
    stdin.readline() # blank line before a testcase
    n, m = [int(x) for x in stdin.readline().strip().split(" ")]
    edges = [ ]
    for i in range(m):
        x, y, dist, ascent = [int(x) for x in stdin.readline().strip().split(" ")]
        assert ascent >= 0
        assert dist >= 0
        edges.append((x-1, y-1, dist, ascent))

    perm = topsort(n,edges)
    relabel = { perm[i]:i for i in range(n) }
    for i in range(m):
        edges[i] = ( relabel[edges[i][0]], relabel[edges[i][1]], edges[i][2], edges[i][3] )
        assert edges[i][0] < edges[i][1]
    return n, edges

def intersect(dist1, dist2):
    """Intersect two (distance,ascent) pairs
    """
    d1, a1 = dist1
    d2, a2 = dist2
    # solve x * d1 + (1-x) * a1 == x*d2 + (1-x) * a2
    # solve a1-a2 == x* (d2 - d1 + a1 - a2)
    return  (a1 - a2) * 1.0 / (d2 - d1 + a1 - a2)


def union(data1, data2):
    """Union two sets of solutions, discarding all too long paths
    """
    stack = []
    for dist_asc in sorted(data1 + data2):
        if not stack:
            stack.append(dist_asc)
            continue
        if dist_asc[1] >= stack[-1][1]:
            continue # worse than the last line on the stack => not needed
        while len(stack) >= 2:
            x1 = intersect(stack[-1], stack[-2])
            x2 = intersect(stack[-1], dist_asc)
            if x2 < x1:
                break
            stack.pop() # previous line is not needed
        stack.append(dist_asc)
    return stack


def solve_testcase():
    n, edges = read_testcase()
    best = [[] for i in range(n)]

    best[0] = [(0, 0)]

    for src, dst, distance, ascent in sorted(edges):
        tmp = [(distance + d, ascent + a) for d, a in best[src]]
        best[dst] = union(best[dst], tmp)

    result = best[n-1]
    if len(result) == 0: result.append(None)
    print len(result) - 1,
    for dist_asc1, dist_asc2 in reversed(zip(result[:-1], result[1:])):
        print "%.12lf" % intersect(dist_asc1, dist_asc2),
    print



if __name__ == "__main__":
    testcases = int(stdin.readline().strip())
    for testcase in range(testcases):
        solve_testcase()




